@@ -113,17 +113,14 @@ geojson_datasource::geojson_datasource(parameters const& params)
     desc_(geojson_datasource::name(),
           *params.get<std::string>("encoding","utf-8")),
     filename_(),
-    inline_string_(),
+    from_inline_string_(false),
     extent_(),
     features_(),
-    tree_(nullptr)
+    tree_(nullptr),
+    num_features_to_query_(*params.get<mapnik::value_integer>("num_features_to_query",5))
 {
     boost::optional<std::string> inline_string = params.get<std::string>("inline");
-    if (inline_string)
-    {
-        inline_string_ = *inline_string;
-    }
-    else
+    if (!inline_string)
     {
         boost::optional<std::string> file = params.get<std::string>("file");
         if (!file) throw mapnik::datasource_exception("GeoJSON Plugin: missing <file> parameter");
@@ -136,10 +133,11 @@ geojson_datasource::geojson_datasource(parameters const& params)
         has_disk_index_ = mapnik::util::exists(filename_ + ".index");
     }
 
-    if (!inline_string_.empty())
+    if (inline_string)
     {
-        char const* start = inline_string_.c_str();
-        char const* end = start + inline_string_.size();
+        from_inline_string_ = true;
+        char const* start = inline_string->c_str();
+        char const* end = start + inline_string->size();
         parse_geojson(start, end);
     }
     else if (has_disk_index_)
@@ -151,7 +149,7 @@ geojson_datasource::geojson_datasource(parameters const& params)
         cache_features_ = *params.get<mapnik::boolean_type>("cache_features", true);
 #if !defined(MAPNIK_MEMORY_MAPPED_FILE)
         mapnik::util::file file(filename_);
-        if (!file.open())
+        if (!file)
         {
             throw mapnik::datasource_exception("GeoJSON Plugin: could not open: '" + filename_ + "'");
         }
@@ -161,14 +159,6 @@ geojson_datasource::geojson_datasource(parameters const& params)
         std::fread(&file_buffer[0], file.size(), 1, file.get());
         char const* start = file_buffer.c_str();
         char const* end = start + file_buffer.length();
-        if (cache_features_)
-        {
-            parse_geojson(start, end);
-        }
-        else
-        {
-            initialise_index(start, end);
-        }
 #else
         boost::optional<mapnik::mapped_region_ptr> mapped_region =
             mapnik::mapped_memory_cache::instance().find(filename_, false);
@@ -179,6 +169,7 @@ geojson_datasource::geojson_datasource(parameters const& params)
 
         char const* start = reinterpret_cast<char const*>((*mapped_region)->get_address());
         char const* end = start + (*mapped_region)->get_size();
+#endif
         if (cache_features_)
         {
             parse_geojson(start, end);
@@ -187,17 +178,32 @@ geojson_datasource::geojson_datasource(parameters const& params)
         {
             initialise_index(start, end);
         }
-#endif
     }
 }
 
 namespace {
+using box_type = box2d<double>;
+using boxes_type = std::vector<std::pair<box_type, std::pair<std::size_t, std::size_t>>>;
 using base_iterator_type = char const*;
 const mapnik::transcoder geojson_datasource_static_tr("utf8");
 const mapnik::json::feature_collection_grammar<base_iterator_type,mapnik::feature_impl> geojson_datasource_static_fc_grammar(geojson_datasource_static_tr);
 const mapnik::json::feature_grammar_callback<base_iterator_type,mapnik::feature_impl> geojson_datasource_static_feature_callback_grammar(geojson_datasource_static_tr);
 const mapnik::json::feature_grammar<base_iterator_type, mapnik::feature_impl> geojson_datasource_static_feature_grammar(geojson_datasource_static_tr);
-const mapnik::json::extract_bounding_box_grammar<base_iterator_type> geojson_datasource_static_bbox_grammar;
+const mapnik::json::extract_bounding_box_grammar<base_iterator_type, boxes_type> geojson_datasource_static_bbox_grammar;
+}
+
+void geojson_datasource::initialise_descriptor(mapnik::feature_ptr const& feature)
+{
+    for ( auto const& kv : *feature)
+    {
+        auto const& name = std::get<0>(kv);
+        if (!desc_.has_name(name))
+        {
+            desc_.add_descriptor(mapnik::attribute_descriptor(name,
+                                                              mapnik::util::apply_visitor(attr_value_converter(),
+                                                                                          std::get<1>(kv))));
+        }
+    }
 }
 
 void geojson_datasource::initialise_disk_index(std::string const& filename)
@@ -213,11 +219,11 @@ void geojson_datasource::initialise_disk_index(std::string const& filename)
     std::vector<value_type> positions;
     mapnik::util::spatial_index<value_type,
                                 mapnik::filter_in_box,
-                                std::ifstream>::query_first_n(filter, index, positions, 5);
+                                std::ifstream>::query_first_n(filter, index, positions, num_features_to_query_);
 
     mapnik::util::file file(filename_);
-    if (!file.open()) throw mapnik::datasource_exception("GeoJSON Plugin: could not open: '" + filename_ + "'");
-
+    if (!file) throw mapnik::datasource_exception("GeoJSON Plugin: could not open: '" + filename_ + "'");
+    mapnik::context_ptr ctx = std::make_shared<mapnik::context_type>();
     for (auto const& pos : positions)
     {
         std::fseek(file.get(), pos.first, SEEK_SET);
@@ -226,8 +232,7 @@ void geojson_datasource::initialise_disk_index(std::string const& filename)
         std::fread(record.data(), pos.second, 1, file.get());
         auto const* start = record.data();
         auto const*  end = start + record.size();
-        mapnik::context_ptr ctx = std::make_shared<mapnik::context_type>();
-        mapnik::feature_ptr feature(mapnik::feature_factory::create(ctx,1));
+        mapnik::feature_ptr feature(mapnik::feature_factory::create(ctx, -1));
         using namespace boost::spirit;
         standard::space_type space;
         if (!boost::spirit::qi::phrase_parse(start, end,
@@ -236,23 +241,15 @@ void geojson_datasource::initialise_disk_index(std::string const& filename)
         {
             throw std::runtime_error("Failed to parse geojson feature");
         }
-        for ( auto const& kv : *feature)
-        {
-            auto const& name = std::get<0>(kv);
-            if (!desc_.has_name(name))
-            {
-                desc_.add_descriptor(mapnik::attribute_descriptor(name,
-                                                                  mapnik::util::apply_visitor(attr_value_converter(),
-                                                                                              std::get<1>(kv))));
-            }
-        }
+        initialise_descriptor(feature);
     }
+    desc_.order_by_name();
 }
 
 template <typename Iterator>
 void geojson_datasource::initialise_index(Iterator start, Iterator end)
 {
-    mapnik::json::boxes_type boxes;
+    boxes_type boxes;
     boost::spirit::standard::space_type space;
     Iterator itr = start;
     if (!boost::spirit::qi::phrase_parse(itr, end, (geojson_datasource_static_bbox_grammar)(boost::phoenix::ref(boxes)) , space))
@@ -268,7 +265,7 @@ void geojson_datasource::initialise_index(Iterator start, Iterator end)
                                                  space);
         if (!result || itr != end)
         {
-            if (!inline_string_.empty()) throw mapnik::datasource_exception("geojson_datasource: Failed to parse GeoJSON file from in-memory string");
+            if (from_inline_string_) throw mapnik::datasource_exception("geojson_datasource: Failed to parse GeoJSON file from in-memory string");
             else throw mapnik::datasource_exception("geojson_datasource: Failed to parse GeoJSON file '" + filename_ + "'");
         }
 
@@ -285,20 +282,18 @@ void geojson_datasource::initialise_index(Iterator start, Iterator end)
                 if (geometry_index == 0)
                 {
                     extent_ = box;
-                    for ( auto const& kv : *f)
-                    {
-                        desc_.add_descriptor(mapnik::attribute_descriptor(std::get<0>(kv),
-                                                                          mapnik::util::apply_visitor(attr_value_converter(),
-                                                                                                      std::get<1>(kv))));
-                    }
                 }
                 else
                 {
                     extent_.expand_to_include(box);
                 }
                 values.emplace_back(box, std::make_pair(geometry_index,0));
+
+            }
+            if (geometry_index++ < num_features_to_query_)
+            {
+                initialise_descriptor(f);
             }
-            ++geometry_index;
         }
         // packing algorithm
         tree_ = std::make_unique<spatial_index_type>(values);
@@ -308,18 +303,20 @@ void geojson_datasource::initialise_index(Iterator start, Iterator end)
         // bulk insert initialise r-tree
         tree_ = std::make_unique<spatial_index_type>(boxes);
         // calculate total extent
+        std::size_t feature_count = 0;
+        mapnik::context_ptr ctx = std::make_shared<mapnik::context_type>();
         for (auto const& item : boxes)
         {
             auto const& box = std::get<0>(item);
             auto const& geometry_index = std::get<1>(item);
-            if (!extent_.valid())
+            if (!extent_.valid()) extent_ = box;
+            else extent_.expand_to_include(box);
+            if (feature_count++ < num_features_to_query_)
             {
-                extent_ = box;
-                // parse first feature to extract attributes schema.
+                // parse first N features to extract attributes schema.
                 // NOTE: this doesn't yield correct answer for geoJSON in general, just an indication
                 Iterator itr2 = start + geometry_index.first;
                 Iterator end2 = itr2 + geometry_index.second;
-                mapnik::context_ptr ctx = std::make_shared<mapnik::context_type>();
                 mapnik::feature_ptr feature(mapnik::feature_factory::create(ctx,-1)); // temp feature
                 if (!boost::spirit::qi::phrase_parse(itr2, end2,
                                                      (geojson_datasource_static_feature_grammar)(boost::phoenix::ref(*feature)), space)
@@ -327,19 +324,12 @@ void geojson_datasource::initialise_index(Iterator start, Iterator end)
                 {
                     throw std::runtime_error("Failed to parse geojson feature");
                 }
-                for ( auto const& kv : *feature)
-                {
-                    desc_.add_descriptor(mapnik::attribute_descriptor(std::get<0>(kv),
-                                                                      mapnik::util::apply_visitor(attr_value_converter(),
-                                                                                                  std::get<1>(kv))));
-                }
-            }
-            else
-            {
-                extent_.expand_to_include(box);
+
+                initialise_descriptor(feature);
             }
         }
     }
+    desc_.order_by_name();
 }
 
 template <typename Iterator>
@@ -356,7 +346,7 @@ void geojson_datasource::parse_geojson(Iterator start, Iterator end)
     try
     {
         bool result = boost::spirit::qi::phrase_parse(itr, end, (geojson_datasource_static_fc_grammar)
-                                                      (boost::phoenix::ref(ctx),boost::phoenix::ref(start_id), boost::phoenix::ref(callback)),
+                                                      (boost::phoenix::ref(ctx), boost::phoenix::ref(start_id), boost::phoenix::ref(callback)),
                                                       space);
         if (!result || itr != end)
         {
@@ -367,7 +357,7 @@ void geojson_datasource::parse_geojson(Iterator start, Iterator end)
                                                      space);
             if (!result || itr != end)
             {
-                if (!inline_string_.empty()) throw mapnik::datasource_exception("geojson_datasource: Failed parse GeoJSON file from in-memory string");
+                if (from_inline_string_) throw mapnik::datasource_exception("geojson_datasource: Failed parse GeoJSON file from in-memory string");
                 else throw mapnik::datasource_exception("geojson_datasource: Failed parse GeoJSON file '" + filename_ + "'");
             }
         }
@@ -381,7 +371,7 @@ void geojson_datasource::parse_geojson(Iterator start, Iterator end)
                                                       space);
         if (!result || itr != end)
         {
-            if (!inline_string_.empty()) throw mapnik::datasource_exception("geojson_datasource: Failed parse GeoJSON file from in-memory string");
+            if (from_inline_string_) throw mapnik::datasource_exception("geojson_datasource: Failed parse GeoJSON file from in-memory string");
             else throw mapnik::datasource_exception("geojson_datasource: Failed parse GeoJSON file '" + filename_ + "'");
         }
     }
@@ -399,12 +389,7 @@ void geojson_datasource::parse_geojson(Iterator start, Iterator end)
             if (geometry_index == 0)
             {
                 extent_ = box;
-                for ( auto const& kv : *f)
-                {
-                    desc_.add_descriptor(mapnik::attribute_descriptor(std::get<0>(kv),
-                                                                      mapnik::util::apply_visitor(attr_value_converter(),
-                                                                                                  std::get<1>(kv))));
-                }
+
             }
             else
             {
@@ -412,6 +397,10 @@ void geojson_datasource::parse_geojson(Iterator start, Iterator end)
             }
             values.emplace_back(box, std::make_pair(geometry_index,0));
         }
+        if (geometry_index < num_features_to_query_)
+        {
+            initialise_descriptor(f);
+        }
         ++geometry_index;
     }
     // packing algorithm
@@ -419,7 +408,7 @@ void geojson_datasource::parse_geojson(Iterator start, Iterator end)
 
 }
 
-geojson_datasource::~geojson_datasource() { }
+geojson_datasource::~geojson_datasource() {}
 
 const char * geojson_datasource::name()
 {
@@ -454,12 +443,12 @@ boost::optional<mapnik::datasource_geometry_t> geojson_datasource::get_geometry_
         std::vector<value_type> positions;
         mapnik::util::spatial_index<value_type,
                                     mapnik::filter_in_box,
-                                    std::ifstream>::query_first_n(filter, index, positions, 5);
+                                    std::ifstream>::query_first_n(filter, index, positions, num_features_to_query_);
 
         mapnik::util::file file(filename_);
 
-        if (!file.open()) throw mapnik::datasource_exception("GeoJSON Plugin: could not open: '" + filename_ + "'");
-
+        if (!file) throw mapnik::datasource_exception("GeoJSON Plugin: could not open: '" + filename_ + "'");
+        mapnik::context_ptr ctx = std::make_shared<mapnik::context_type>();
         for (auto const& pos : positions)
         {
             std::fseek(file.get(), pos.first, SEEK_SET);
@@ -468,7 +457,6 @@ boost::optional<mapnik::datasource_geometry_t> geojson_datasource::get_geometry_
             std::fread(record.data(), pos.second, 1, file.get());
             auto const* start = record.data();
             auto const*  end = start + record.size();
-            mapnik::context_ptr ctx = std::make_shared<mapnik::context_type>();
             mapnik::feature_ptr feature(mapnik::feature_factory::create(ctx, -1)); // temp feature
             using namespace boost::spirit;
             standard::space_type space;
@@ -494,7 +482,7 @@ boost::optional<mapnik::datasource_geometry_t> geojson_datasource::get_geometry_
     else if (cache_features_)
     {
         unsigned num_features = features_.size();
-        for (unsigned i = 0; i < num_features && i < 5; ++i)
+        for (unsigned i = 0; i < num_features && i < num_features_to_query_; ++i)
         {
             result = mapnik::util::to_ds_type(features_[i]->get_geometry());
             if (result)
@@ -512,14 +500,14 @@ boost::optional<mapnik::datasource_geometry_t> geojson_datasource::get_geometry_
     else
     {
         mapnik::util::file file(filename_);
-        if (!file.open())
+        if (!file)
         {
             throw mapnik::datasource_exception("GeoJSON Plugin: could not open: '" + filename_ + "'");
         }
         auto itr = tree_->qbegin(boost::geometry::index::intersects(extent_));
         auto end = tree_->qend();
         mapnik::context_ptr ctx = std::make_shared<mapnik::context_type>();
-        for (std::size_t count = 0; itr !=end &&  count < 5; ++itr,++count)
+        for (std::size_t count = 0; itr !=end &&  count < num_features_to_query_; ++itr,++count)
         {
             geojson_datasource::item_type const& item = *itr;
             std::size_t file_offset = item.second.first;
@@ -589,8 +577,8 @@ mapnik::featureset_ptr geojson_datasource::features(mapnik::query const& q) cons
         }
 
     }
-    // otherwise return an empty featureset pointer
-    return mapnik::featureset_ptr();
+    // otherwise return an empty featureset
+    return mapnik::make_invalid_featureset();
 }
 
 mapnik::featureset_ptr geojson_datasource::features_at_point(mapnik::coord2d const& pt, double tol) const
