@@ -26,24 +26,6 @@
 namespace mapnik
 {
 
-static const char * gradient_strings[] = {
-    "no-gradient",
-    "linear",
-    "radial",
-    ""
-};
-
-IMPLEMENT_ENUM( gradient_e, gradient_strings )
-
-static const char * gradient_unit_strings[] = {
-    "user-space-on-use",
-    "user-space-on-use-bounding-box",
-    "object-bounding-box",
-    ""
-};
-
-IMPLEMENT_ENUM( gradient_unit_e, gradient_unit_strings )
-
 gradient::gradient()
 : transform_(),
   x1_(0),
