@@ -63,7 +63,7 @@ CONFIG_MAPNIK_LIBNAME='%(mapnik_libname)s'
 CONFIG_MAPNIK_LIBPATH="%(mapnik_libpath)s"
 CONFIG_DEP_LIBS='%(dep_libs)s'
 CONFIG_MAPNIK_LDFLAGS="%(ldflags)s"
-CONFIG_MAPNIK_INCLUDE="${CONFIG_PREFIX}/include -I${CONFIG_PREFIX}/include/mapnik/agg"
+CONFIG_MAPNIK_INCLUDE="${CONFIG_PREFIX}/include -I${CONFIG_PREFIX}/include/mapnik/agg -I${CONFIG_PREFIX}/include/mapnik"
 CONFIG_DEP_INCLUDES="%(dep_includes)s"
 CONFIG_CXXFLAGS="%(cxxflags)s"
 CONFIG_CXX='%(cxx)s'
@@ -80,7 +80,15 @@ def write_config(configuration,template,config_file):
         os.chmod(config_file,0755)
     except: pass
 
-cxxflags = ' '.join(config_env['LIBMAPNIK_CXXFLAGS'])
+
+cxxflags_raw = config_env['LIBMAPNIK_CXXFLAGS'];
+# strip clang specific flags to avoid breaking gcc
+# while it is not recommended to mix compilers, this nevertheless
+# makes it easier to compile apps with gcc and mapnik-config against mapnik built with clang
+to_remove = ['-Wno-unsequenced','-Wno-unknown-warning-option','-Wtautological-compare','-Wheader-hygiene']
+cxxflags_cleaned = [item for item in config_env['LIBMAPNIK_CXXFLAGS'] if item not in to_remove];
+
+cxxflags = ' '.join(cxxflags_cleaned)
 
 defines = ' '.join(config_env['LIBMAPNIK_DEFINES'])
 
@@ -131,11 +139,6 @@ mapnik_bundled_gdal_data = ''
 mapnik_bundled_proj_data = ''
 mapnik_bundled_icu_data = ''
 
-if config_env.get('MAPNIK_BUNDLED_SHARE_DIRECTORY'):
-    mapnik_bundled_gdal_data = 'lib/mapnik/share/gdal'
-    mapnik_bundled_proj_data = 'lib/mapnik/share/proj'
-    mapnik_bundled_icu_data = 'lib/mapnik/share/icu'
-
 configuration = {
     "git_revision": git_revision,
     "git_describe": git_describe,
