@@ -26,9 +26,10 @@
 #include <string>
 #include <mapnik/util/fs.hpp>
 #include <mapnik/quad_tree.hpp>
+#include <mapnik/geometry_envelope.hpp>
 #include "shapefile.hpp"
 #include "shape_io.hpp"
-
+#include "shape_index_featureset.hpp"
 #pragma GCC diagnostic push
 #include <mapnik/warning_ignore.hpp>
 #include <boost/algorithm/string.hpp>
@@ -44,8 +45,9 @@ int main (int argc,char** argv)
     namespace po = boost::program_options;
 
     bool verbose=false;
-    unsigned int depth=DEFAULT_DEPTH;
-    double ratio=DEFAULT_RATIO;
+    bool index_parts = false;
+    unsigned int depth = DEFAULT_DEPTH;
+    double ratio = DEFAULT_RATIO;
     std::vector<std::string> shape_files;
 
     try
@@ -54,6 +56,7 @@ int main (int argc,char** argv)
         desc.add_options()
             ("help,h", "produce usage message")
             ("version,V","print version string")
+            ("index-parts","index individual shape parts (default: no)")
             ("verbose,v","verbose output")
             ("depth,d", po::value<unsigned int>(), "max tree depth\n(default 8)")
             ("ratio,r",po::value<double>(),"split ratio (default 0.55)")
@@ -69,18 +72,22 @@ int main (int argc,char** argv)
         if (vm.count("version"))
         {
             std::clog << "version 0.3.0" <<std::endl;
-            return 1;
+            return EXIT_FAILURE;
         }
 
         if (vm.count("help"))
         {
             std::clog << desc << std::endl;
-            return 1;
+            return EXIT_FAILURE;
         }
         if (vm.count("verbose"))
         {
             verbose = true;
         }
+        if (vm.count("index-parts"))
+        {
+            index_parts = true;
+        }
         if (vm.count("depth"))
         {
             depth = vm["depth"].as<unsigned int>();
@@ -98,7 +105,7 @@ int main (int argc,char** argv)
     catch (std::exception const& ex)
     {
         std::clog << "Error: " << ex.what() << std::endl;
-        return -1;
+        return EXIT_FAILURE;
     }
 
     std::clog << "max tree depth:" << depth << std::endl;
@@ -107,7 +114,7 @@ int main (int argc,char** argv)
     if (shape_files.size() == 0)
     {
         std::clog << "no shape files to index" << std::endl;
-        return 0;
+        return EXIT_FAILURE;
     }
     for (auto const& filename : shape_files)
     {
@@ -157,54 +164,103 @@ int main (int argc,char** argv)
         std::clog << "type=" << shape_type << std::endl;
         std::clog << "extent:" << extent << std::endl;
 
+        if (!extent.valid() || std::isnan(extent.width()) || std::isnan(extent.height()))
+        {
+            std::clog << "Invalid extent aborting..." << std::endl;
+            return EXIT_FAILURE;
+        }
         int pos = 50;
         shx.seek(pos * 2);
-        mapnik::quad_tree<int> tree(extent, depth, ratio);
+        mapnik::quad_tree<mapnik::detail::node> tree(extent, depth, ratio);
         int count = 0;
 
-        while (shx.is_good() && pos <= file_length - 4)
+        if (shape_type != shape_io::shape_null)
         {
-            int offset = shx.read_xdr_integer();
-            int content_length = shx.read_xdr_integer();
-            pos += 4;
-            box2d<double> item_ext;
-            shp.seek(offset * 2);
-            int record_number = shp.read_xdr_integer();
-            if (content_length != shp.read_xdr_integer())
+            while (shx.is_good() && pos <= file_length - 4)
             {
-                std::clog << "Content length mismatch for record number " << record_number << std::endl;
-                continue;
-            }
-            shape_type = shp.read_ndr_integer();
+                int offset = shx.read_xdr_integer();
+                int shx_content_length = shx.read_xdr_integer();
+                pos += 4;
+                box2d<double> item_ext;
+                shp.seek(offset * 2);
+                int record_number = shp.read_xdr_integer();
+                int shp_content_length = shp.read_xdr_integer();
+                if (shx_content_length != shp_content_length)
+                {
+                    std::clog << "Content length mismatch for record number " << record_number << std::endl;
+                    continue;
+                }
+                shape_type = shp.read_ndr_integer();
 
-            if (shape_type==shape_io::shape_point
-                || shape_type==shape_io::shape_pointm
-                || shape_type == shape_io::shape_pointz)
-            {
-                double x=shp.read_double();
-                double y=shp.read_double();
-                item_ext=box2d<double>(x,y,x,y);
-            }
-            else
-            {
-                shp.read_envelope(item_ext);
-            }
-            if (verbose)
-            {
-                std::clog << "record number " << record_number << " box=" << item_ext << std::endl;
-            }
-            if (item_ext.valid())
-            {
-                tree.insert(offset * 2,item_ext);
-                ++count;
+                if (shape_type == shape_io::shape_null) continue;
+
+                if (shape_type==shape_io::shape_point
+                    || shape_type==shape_io::shape_pointm
+                    || shape_type == shape_io::shape_pointz)
+                {
+                    double x=shp.read_double();
+                    double y=shp.read_double();
+                    item_ext=box2d<double>(x,y,x,y);
+                }
+                else if (index_parts &&
+                         (shape_type == shape_io::shape_polygon || shape_type == shape_io::shape_polygonm || shape_type == shape_io::shape_polygonz
+                          || shape_type == shape_io::shape_polyline || shape_type == shape_io::shape_polylinem || shape_type == shape_io::shape_polylinez))
+                {
+                    shp.read_envelope(item_ext);
+                    int num_parts = shp.read_ndr_integer();
+                    int num_points = shp.read_ndr_integer();
+                    std::vector<int> parts;
+                    parts.resize(num_parts);
+                    std::for_each(parts.begin(), parts.end(), [&](int & part) { part = shp.read_ndr_integer();});
+                    for (int k = 0; k < num_parts; ++k)
+                    {
+                        int start = parts[k];
+                        int end;
+                        if (k == num_parts - 1) end = num_points;
+                        else end = parts[k + 1];
+
+                        mapnik::geometry::linear_ring<double> ring;
+                        ring.reserve(end - start);
+                        for (int j = start; j < end; ++j)
+                        {
+                            double x = shp.read_double();
+                            double y = shp.read_double();
+                            ring.emplace_back(x, y);
+                        }
+                        item_ext = mapnik::geometry::envelope(ring);
+                        if (item_ext.valid())
+                        {
+                            if (verbose)
+                            {
+                                std::clog << "record number " << record_number << " box=" << item_ext << std::endl;
+                            }
+                            tree.insert(mapnik::detail::node(offset * 2, start, end),item_ext);
+                            ++count;
+                        }
+                    }
+                    item_ext = mapnik::box2d<double>(); //invalid
+                }
+                else
+                {
+                    shp.read_envelope(item_ext);
+                }
+
+                if (item_ext.valid())
+                {
+                    if (verbose)
+                    {
+                        std::clog << "record number " << record_number << " box=" << item_ext << std::endl;
+                    }
+                    tree.insert(mapnik::detail::node(offset * 2,-1,0),item_ext);
+                    ++count;
+                }
             }
         }
 
         if (count > 0)
         {
             std::clog << " number shapes=" << count << std::endl;
-            std::fstream file((shapename+".index").c_str(),
-                              std::ios::in | std::ios::out | std::ios::trunc | std::ios::binary);
+            std::ofstream file((shapename+".index").c_str(), std::ios::trunc | std::ios::binary);
             if (!file)
             {
                 std::clog << "cannot open index file for writing file \""
@@ -227,5 +283,5 @@ int main (int argc,char** argv)
     }
 
     std::clog << "done!" << std::endl;
-    return 0;
+    return EXIT_SUCCESS;
 }
